@@ -10,7 +10,8 @@ use rio_backend::clipboard::{Clipboard, ClipboardType};
 use rio_backend::config::colors::ColorRgb;
 use rio_window::application::ApplicationHandler;
 use rio_window::event::{
-    ElementState, Ime, MouseButton, MouseScrollDelta, StartCause, TouchPhase, WindowEvent,
+    ElementState, Hook, Ime, MouseButton, MouseScrollDelta, StartCause, TouchPhase,
+    WindowEvent,
 };
 use rio_window::event_loop::ActiveEventLoop;
 use rio_window::event_loop::ControlFlow;
@@ -54,9 +55,11 @@ impl Application<'_> {
             event_proxy.clone(),
         );
         let scheduler = Scheduler::new(proxy);
-
         event_loop.listen_device_events(DeviceEvents::Never);
 
+        #[cfg(target_os = "macos")]
+        event_loop.set_confirm_before_quit(config.confirm_before_quit);
+
         Application {
             config,
             event_proxy,
@@ -112,7 +115,14 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
     }
 
     fn new_events(&mut self, event_loop: &ActiveEventLoop, cause: StartCause) {
-        if cause != StartCause::Init {
+        if cause != StartCause::Init
+            && cause != StartCause::CreateWindow
+            && cause != StartCause::MacOSReopen
+        {
+            return;
+        }
+
+        if cause == StartCause::MacOSReopen && !self.router.routes.is_empty() {
             return;
         }
 
@@ -151,9 +161,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     }
 
                     if route_id == route.window.screen.ctx().current_route() {
-                        if self.config.renderer.max_fps == 0 {
-                            route.request_redraw();
-                        } else {
+                        if let Some(limit) = route.window.frame_time_limit {
                             let timer_id = TimerId::new(Topic::RenderRoute, window_id);
                             let event = EventPayload::new(
                                 RioEventType::Rio(RioEvent::Render),
@@ -161,15 +169,10 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                             );
 
                             if !self.scheduler.scheduled(timer_id) {
-                                self.scheduler.schedule(
-                                    event,
-                                    Duration::from_millis(
-                                        1000 / self.config.renderer.max_fps,
-                                    ),
-                                    false,
-                                    timer_id,
-                                );
+                                self.scheduler.schedule(event, limit, false, timer_id);
                             }
+                        } else {
+                            route.request_redraw();
                         }
                     }
                 }
@@ -273,9 +276,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
             RioEventType::Rio(RioEvent::CursorBlinkingChange) => {
                 if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    if route.window.has_frame {
-                        route.request_redraw();
-                    }
+                    route.request_redraw();
                 }
             }
             RioEventType::Rio(RioEvent::CursorBlinkingChangeOnRoute(route_id)) => {
@@ -484,26 +485,9 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
             #[cfg(target_os = "macos")]
             RioEventType::Rio(RioEvent::CloseWindow) => {
-                if let Some(route) = self.router.routes.get_mut(&window_id) {
-                    if route.window.winit_window.num_tabs() > 1 {
-                        self.router.routes.remove(&window_id);
-                    } else {
-                        // In MacOS: Close last tab will work
-                        // leading to hide and run Rio in background
-                        let routes_len = self.router.routes.len();
-                        self.router.routes.remove(&window_id);
-                        // If was the last last window opened then hide the application
-                        if routes_len == 1 {
-                            let config = &self.config;
-                            event_loop.hide_application();
-                            self.router.create_window(
-                                event_loop,
-                                self.event_proxy.clone(),
-                                config,
-                                None,
-                            );
-                        }
-                    }
+                self.router.routes.remove(&window_id);
+                if self.router.routes.is_empty() && !self.config.confirm_before_quit {
+                    event_loop.exit();
                 }
             }
             #[cfg(target_os = "macos")]
@@ -624,6 +608,10 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             WindowEvent::CloseRequested => {
                 self.router.routes.remove(&window_id);
 
+                if self.config.confirm_before_quit {
+                    return;
+                }
+
                 if self.router.routes.is_empty() {
                     event_loop.exit();
                 }
@@ -970,9 +958,7 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
 
                         if route.window.screen.ime.preedit() != preedit.as_ref() {
                             route.window.screen.ime.set_preedit(preedit);
-                            if route.window.has_frame {
-                                route.request_redraw();
-                            }
+                            route.request_redraw();
                         }
                     }
                     Ime::Enabled => {
@@ -992,12 +978,12 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                     route.window.winit_window.set_cursor_visible(true);
                 }
 
-                let has_regained_focus = !route.window.is_focused && focused;
-                route.window.is_focused = focused;
+                // let has_regained_focus = !route.window.is_focused && focused;
+                // route.window.is_focused = focused;
 
-                if has_regained_focus {
-                    route.request_redraw();
-                }
+                // if has_regained_focus {
+                // route.request_redraw();
+                // }
 
                 route.window.screen.on_focus_change(focused);
             }
@@ -1044,9 +1030,6 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             }
 
             WindowEvent::RedrawRequested => {
-                // let start = std::time::Instant::now();
-                route.window.has_updates = false;
-
                 route.window.winit_window.pre_present_notify();
                 match route.path {
                     RoutePath::Assistant => {
@@ -1065,9 +1048,9 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                             .render_dialog("Do you want to leave Rio?");
                     }
                 }
-                // let duration = start.elapsed();
                 // println!("Time elapsed in render() is: {:?}", duration);
                 // }
+
                 event_loop.set_control_flow(ControlFlow::Wait);
             }
             _ => {}
@@ -1085,6 +1068,51 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
         event_loop.set_control_flow(control_flow);
     }
 
+    fn open_config(&mut self, event_loop: &ActiveEventLoop) {
+        self.router.open_config_window(
+            event_loop,
+            self.event_proxy.clone(),
+            &self.config,
+        );
+    }
+
+    fn hook_event(&mut self, _event_loop: &ActiveEventLoop, hook: &Hook) {
+        let window_id = match self.router.get_focused_route() {
+            Some(window_id) => window_id,
+            None => return,
+        };
+
+        let route = match self.router.routes.get_mut(&window_id) {
+            Some(window) => window,
+            None => return,
+        };
+
+        match hook {
+            Hook::Copy => {
+                route.window.screen.copy_selection(ClipboardType::Clipboard);
+            }
+            Hook::Paste => {
+                let content = route
+                    .window
+                    .screen
+                    .clipboard
+                    .borrow_mut()
+                    .get(ClipboardType::Selection);
+                route.window.screen.paste(&content, true);
+            }
+            Hook::CreateTab => {
+                if self.config.navigation.has_navigation_key_bindings() {
+                    route.window.screen.create_tab();
+                }
+            }
+            Hook::CloseTab => {
+                if self.config.navigation.has_navigation_key_bindings() {
+                    route.window.screen.close_tab();
+                }
+            }
+        }
+    }
+
     // Emitted when the event loop is being shut down.
     // This is irreversible - if this event is emitted, it is guaranteed to be the last event that gets emitted.
     // You generally want to treat this as an “do on quit” event.
