@@ -19,7 +19,6 @@ use rio_backend::sugarloaf::{
     Content, FragmentStyle, FragmentStyleDecoration, Graphic, Stretch, Style,
     SugarCursor, Sugarloaf, UnderlineInfo, UnderlineShape, Weight,
 };
-use rio_window::window::Theme;
 use std::collections::HashMap;
 use std::ops::RangeInclusive;
 use std::time::{Duration, Instant};
@@ -47,6 +46,7 @@ pub struct Renderer {
     cursor: Cursor,
     pub selection_range: Option<SelectionRange>,
     pub config_has_blinking_enabled: bool,
+    pub config_blinking_interval: u64,
     term_has_blinking_enabled: bool,
     pub is_blinking: bool,
     ignore_selection_fg_color: bool,
@@ -65,25 +65,11 @@ pub struct Renderer {
 impl Renderer {
     pub fn new(
         config: &Config,
-        current_theme: Option<Theme>,
         font_context: &rio_backend::sugarloaf::font::FontLibrary,
     ) -> Renderer {
         let term_colors = TermColors::default();
         let colors = List::from(&term_colors);
-        let mut named_colors = config.colors;
-
-        if let Some(theme) = current_theme {
-            if let Some(adaptive_colors) = &config.adaptive_colors {
-                match theme {
-                    Theme::Light => {
-                        named_colors = adaptive_colors.light.unwrap_or(named_colors);
-                    }
-                    Theme::Dark => {
-                        named_colors = adaptive_colors.dark.unwrap_or(named_colors);
-                    }
-                }
-            }
-        }
+        let named_colors = config.colors;
 
         let mut dynamic_background =
             (named_colors.background.0, named_colors.background.1, false);
@@ -106,6 +92,7 @@ impl Renderer {
         }
 
         Renderer {
+            config_blinking_interval: config.cursor.blinking_interval.clamp(350, 1200),
             option_as_alt: config.option_as_alt.to_lowercase(),
             is_kitty_keyboard_enabled: config.keyboard.use_kitty_keyboard_protocol,
             is_ime_enabled: false,
