@@ -16,9 +16,10 @@ use rio_window::platform::startup_notify::{
 };
 use rio_window::window::{Window, WindowId};
 use routes::{assistant, RoutePath};
+use rustc_hash::FxHashMap;
 use std::cell::RefCell;
-use std::collections::HashMap;
 use std::rc::Rc;
+use std::time::Duration;
 
 // 𜱭𜱭 unicode is not available yet for all OS
 // https://www.unicode.org/charts/PDF/Unicode-16.0/U160-1CC00.pdf
@@ -140,7 +141,7 @@ impl Route<'_> {
 }
 
 pub struct Router<'a> {
-    pub routes: HashMap<WindowId, Route<'a>>,
+    pub routes: FxHashMap<WindowId, Route<'a>>,
     propagated_report: Option<RioError>,
     pub font_library: Box<rio_backend::sugarloaf::font::FontLibrary>,
     pub config_route: Option<WindowId>,
@@ -167,7 +168,7 @@ impl Router<'_> {
         let clipboard = Rc::new(RefCell::new(clipboard));
 
         Router {
-            routes: HashMap::default(),
+            routes: FxHashMap::default(),
             propagated_report,
             config_route: None,
             font_library: Box::new(font_library),
@@ -189,6 +190,20 @@ impl Router<'_> {
         }
     }
 
+    #[inline]
+    pub fn get_focused_route(&self) -> Option<WindowId> {
+        self.routes
+            .iter()
+            .find_map(|(key, val)| {
+                if val.window.winit_window.has_focus() {
+                    Some(key)
+                } else {
+                    None
+                }
+            })
+            .copied()
+    }
+
     pub fn open_config_window(
         &mut self,
         event_loop: &ActiveEventLoop,
@@ -309,8 +324,7 @@ impl Router<'_> {
 pub struct RouteWindow<'a> {
     pub is_focused: bool,
     pub is_occluded: bool,
-    pub has_frame: bool,
-    pub has_updates: bool,
+    pub frame_time_limit: Option<Duration>,
     pub winit_window: Window,
     pub screen: Screen<'a>,
     #[cfg(target_os = "macos")]
@@ -366,10 +380,17 @@ impl<'a> RouteWindow<'a> {
         )
         .expect("Screen not created");
 
+        let frame_time_limit = if config.renderer.max_fps == 0 {
+            None
+        } else {
+            Some(Duration::from_millis(
+                1000 / config.renderer.max_fps.clamp(1, 1000),
+            ))
+        };
+
         Self {
-            has_frame: true,
-            has_updates: true,
-            is_focused: false,
+            frame_time_limit,
+            is_focused: true,
             is_occluded: false,
             winit_window,
             screen,
