@@ -18,9 +18,9 @@ pub fn default_cursor_interval() -> u64 {
 #[inline]
 pub fn default_max_fps() -> u64 {
     if cfg!(target_os = "macos") {
-        120
+        240
     } else {
-        60
+        90
     }
 }
 
@@ -218,7 +218,7 @@ pub fn default_config_file_content() -> String {
 # [cursor]
 # shape = 'block'
 # blinking = false
-# blinking-interval = 800 
+# blinking-interval = 800
 
 # Editor
 #
@@ -436,7 +436,7 @@ pub fn default_config_file_content() -> String {
 # Bindings
 #
 # Create custom Key bindings for Rio terminal
-# More information in: https://raphamorim.io/rio/docs/key-bindings#custom-key-bindings
+# More information in: https://raphamorim.io/rio/docs/config/bindings
 #
 # Example:
 # [bindings]
