@@ -1,3 +1,7 @@
+use crate::platform_impl::platform::menu::menu_item;
+use objc2::sel;
+use objc2_app_kit::NSMenu;
+use objc2_foundation::ns_string;
 use std::cell::{Cell, RefCell};
 use std::collections::VecDeque;
 use std::mem;
@@ -21,10 +25,19 @@ use super::observer::{EventLoopWaker, RunLoop};
 use super::window::WinitWindow;
 use super::{menu, WindowId, DEVICE_ID};
 use crate::dpi::PhysicalSize;
-use crate::event::{DeviceEvent, Event, InnerSizeWriter, StartCause, WindowEvent};
+use crate::event::{DeviceEvent, Event, Hook, InnerSizeWriter, StartCause, WindowEvent};
 use crate::event_loop::{ActiveEventLoop as RootActiveEventLoop, ControlFlow};
 use crate::window::WindowId as RootWindowId;
 
+#[repr(u64)]
+#[derive(Copy, Clone, PartialEq)]
+pub enum NSApplicationTerminateReply {
+    Cancel = 0,
+    Now = 1,
+    #[allow(unused)]
+    Later = 2,
+}
+
 #[derive(Debug)]
 struct Policy(NSApplicationActivationPolicy);
 
@@ -38,6 +51,7 @@ impl Default for Policy {
 pub(super) struct State {
     activation_policy: Policy,
     default_menu: bool,
+    set_confirm_before_quit: Cell<bool>,
     activate_ignoring_other_apps: bool,
     event_handler: EventHandler,
     stop_on_launch: Cell<bool>,
@@ -77,6 +91,145 @@ declare_class!(
     unsafe impl NSObjectProtocol for ApplicationDelegate {}
 
     unsafe impl NSApplicationDelegate for ApplicationDelegate {
+        #[method(applicationShouldTerminate:)]
+        fn should_terminate(&self, _sender: Option<&AnyObject>) -> u64 {
+            if !self.ivars().set_confirm_before_quit.get() {
+                return NSApplicationTerminateReply::Now as u64;
+            }
+
+            use objc::runtime::Object;
+            use objc::msg_send;
+            use objc::sel;
+            use objc::class;
+            use objc::sel_impl;
+            unsafe {
+                let panel: *mut Object = msg_send![class!(NSAlert), new];
+
+                let prompt = "All sessions will be closed";
+                let title = "Quit Rio terminal?";
+                let yes = "Yes";
+                let no = "No";
+                let cancel = "Cancel";
+
+                let prompt_string: *mut Object = msg_send![class!(NSString), alloc];
+                let prompt_allocated_string: *mut Object = msg_send![prompt_string, initWithBytes:prompt.as_ptr() length:prompt.len() encoding:4];
+
+                let title_string: *mut Object = msg_send![class!(NSString), alloc];
+                let title_allocated_string: *mut Object = msg_send![title_string, initWithBytes:title.as_ptr() length:title.len() encoding:4];
+
+                let yes_string: *mut Object = msg_send![class!(NSString), alloc];
+                let yes_allocated_string: *mut Object = msg_send![yes_string, initWithBytes:yes.as_ptr() length:yes.len() encoding:4];
+
+                let no_string: *mut Object = msg_send![class!(NSString), alloc];
+                let no_allocated_string: *mut Object = msg_send![no_string, initWithBytes:no.as_ptr() length:no.len() encoding:4];
+
+                let cancel_string: *mut Object = msg_send![class!(NSString), alloc];
+                let cancel_allocated_string: *mut Object = msg_send![cancel_string, initWithBytes:cancel.as_ptr() length:cancel.len() encoding:4];
+
+                let _: () = msg_send![panel, setMessageText: title_allocated_string];
+                let _: () = msg_send![panel, setInformativeText: prompt_allocated_string];
+                let _: () = msg_send![panel, addButtonWithTitle: yes_allocated_string];
+                let _: () = msg_send![panel, addButtonWithTitle: no_allocated_string];
+                let _: () = msg_send![panel, addButtonWithTitle: cancel_allocated_string];
+                let response: std::ffi::c_long = msg_send![panel, runModal];
+                match response {
+                    1000 => NSApplicationTerminateReply::Now as u64,
+                    1001 => NSApplicationTerminateReply::Cancel as u64,
+                    _ => NSApplicationTerminateReply::Cancel as u64,
+                }
+            }
+        }
+
+        #[method(applicationDockMenu:)]
+        fn dock_menu(&self, _sender: Option<&AnyObject>) -> *mut NSMenu {
+            let mtm = MainThreadMarker::from(self);
+
+            let menubar = NSMenu::new(mtm);
+            let new_window_item_title = ns_string!("New Window");
+            let new_window_item = menu_item(
+                mtm,
+                new_window_item_title,
+                Some(sel!(rioCreateWindow:)),
+                None,
+            );
+            let new_tab_item_title = ns_string!("New Tab");
+            let new_tab_item = menu_item(
+                mtm,
+                new_tab_item_title,
+                Some(sel!(rioCreateTab:)),
+                None,
+            );
+            menubar.addItem(&new_window_item);
+            menubar.addItem(&new_tab_item);
+            Retained::<NSMenu>::autorelease_return(menubar)
+        }
+
+        #[method(rioCreateWindow:)]
+        fn create_window(
+            &self,
+            _sender: Option<&AnyObject>,
+        ) {
+            if self.is_launched() {
+                self.dispatch_create_window_event();
+            }
+        }
+
+        #[method(copy:)]
+        fn copy(&self, _sender: Option<&AnyObject>) {
+            if self.is_launched() {
+                self.dispatch_hook(Hook::Copy);
+            }
+        }
+
+        #[method(paste:)]
+        fn paste(&self, _sender: Option<&AnyObject>) {
+            if self.is_launched() {
+                self.dispatch_hook(Hook::Paste);
+            }
+        }
+
+        #[method(rioCreateTab:)]
+        fn create_tab(&self, _sender: Option<&AnyObject>) {
+            if self.is_launched() {
+                self.dispatch_hook(Hook::CreateTab);
+            }
+        }
+
+        #[method(rioCloseTab:)]
+        fn close_tab(&self, _sender: Option<&AnyObject>) {
+            if self.is_launched() {
+                self.dispatch_hook(Hook::CloseTab);
+            }
+        }
+
+        #[method(openConfig:)]
+        fn open_configuration(
+            &self,
+            _sender: Option<&AnyObject>,
+        ) {
+            if self.is_launched() {
+                self.dispatch_open_configuration();
+            }
+        }
+
+        #[method(applicationShouldHandleReopen:hasVisibleWindows:)]
+        fn should_handle_reopen(&self,
+            _sender: Option<&AnyObject>,
+            has_open_windows: bool,
+        ) -> bool {
+            if self.is_launched() && !has_open_windows {
+                self.dispatch_application_reopen();
+                true
+            } else {
+                false
+            }
+        }
+
+        #[method(applicationShouldTerminateAfterLastWindowClosed:)]
+        fn should_terminate_after_last_window_closed(&self, _sender: Option<&AnyObject>) -> bool {
+            false
+        }
+
         // NOTE: This will, globally, only be run once, no matter how many
         // `EventLoop`s the user creates.
         #[method(applicationDidFinishLaunching:)]
@@ -91,7 +244,6 @@ declare_class!(
             // menu bar is initially unresponsive on macOS 10.15.
             app.setActivationPolicy(self.ivars().activation_policy.0);
 
-            window_activation_hack(&app);
             #[allow(deprecated)]
             app.activateIgnoringOtherApps(self.ivars().activate_ignoring_other_apps);
 
@@ -162,6 +314,21 @@ impl ApplicationDelegate {
             activate_ignoring_other_apps,
             ..Default::default()
         });
+
+        // let this: Retained<Self> = unsafe { msg_send_id![super(this), init] };
+
+        // let workspace = &unsafe { NSWorkspace::sharedWorkspace() };
+        // let workspace_center = &unsafe { workspace.notificationCenter() };
+        // unsafe {
+        //     workspace_center.addObserver_selector_name_object(
+        //         &this,
+        //         sel!(applicationDidUnhide:),
+        //         // Some(ns_string!("NSWorkspaceDidActivateApplicationNotification")),
+        //         Some(ns_string!("NSWorkspaceDidUnhideApplicationNotification")),
+        //         Some(workspace),
+        //     )
+        // }
+
         unsafe { msg_send_id![super(this), init] }
     }
 
@@ -187,6 +354,12 @@ impl ApplicationDelegate {
         self.ivars().event_handler.set(handler, closure)
     }
 
+    /// Place the event handler in the application delegate for the duration
+    /// of the given closure.
+    pub fn set_confirm_before_quit(&self, confirmation: bool) {
+        self.ivars().set_confirm_before_quit.set(confirmation)
+    }
+
     /// If `pump_events` is called to progress the event loop then we
     /// bootstrap the event loop via `-[NSApplication run]` but will use
     /// `CFRunLoopRunInMode` for subsequent calls to `pump_events`.
@@ -319,6 +492,10 @@ impl ApplicationDelegate {
             .handle_event(event, &ActiveEventLoop::new_root(self.retain()))
     }
 
+    pub fn dispatch_application_reopen(&self) {
+        self.handle_event(Event::NewEvents(StartCause::MacOSReopen));
+    }
+
     /// dispatch `NewEvents(Init)` + `Resumed`
     pub fn dispatch_init_events(&self) {
         self.handle_event(Event::NewEvents(StartCause::Init));
@@ -327,6 +504,18 @@ impl ApplicationDelegate {
         self.handle_event(Event::Resumed);
     }
 
+    pub fn dispatch_create_window_event(&self) {
+        self.handle_event(Event::NewEvents(StartCause::CreateWindow));
+    }
+
+    pub fn dispatch_hook(&self, hook: Hook) {
+        self.handle_event(Event::HookEvent(hook));
+    }
+
+    pub fn dispatch_open_configuration(&self) {
+        self.handle_event(Event::OpenConfig);
+    }
+
     pub fn open_urls(&self, urls: Vec<String>) {
         self.handle_event(Event::Opened { urls });
     }
@@ -500,26 +689,3 @@ fn min_timeout(a: Option<Instant>, b: Option<Instant>) -> Option<Instant> {
         b.map_or(Some(a_timeout), |b_timeout| Some(a_timeout.min(b_timeout)))
     })
 }
-
-/// A hack to make activation of multiple windows work when creating them before
-/// `applicationDidFinishLaunching:` / `Event::Event::NewEvents(StartCause::Init)`.
-///
-/// Alternative to this would be the user calling `window.set_visible(true)` in
-/// `StartCause::Init`.
-///
-/// If this becomes too bothersome to maintain, it can probably be removed
-/// without too much damage.
-fn window_activation_hack(app: &NSApplication) {
-    // TODO: Proper ordering of the windows
-    app.windows().into_iter().for_each(|window| {
-        // Call `makeKeyAndOrderFront` if it was called on the window in `WinitWindow::new`
-        // This way we preserve the user's desired initial visibility status
-        // TODO: Also filter on the type/"level" of the window, and maybe other things?
-        if window.isVisible() {
-            tracing::trace!("Activating visible window");
-            window.makeKeyAndOrderFront(None);
-        } else {
-            tracing::trace!("Skipping activating invisible window");
-        }
-    })
-}
