@@ -308,6 +308,10 @@ impl RichTextBrush {
         }
     }
 
+    pub fn reset(&mut self) {
+        self.glyphs = GlyphCache::new();
+    }
+
     #[inline]
     pub fn render<'pass>(
         &'pass mut self,
