@@ -1,12 +1,18 @@
 use crate::font::DEFAULT_FONT_FAMILY;
 use serde::{Deserialize, Serialize};
 
+#[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
+pub enum SugarloafFontStyle {
+    Normal,
+    Italic,
+}
+
 #[derive(Debug, Serialize, Deserialize, PartialEq, Clone)]
 pub struct SugarloafFont {
     #[serde(default = "default_font_family")]
     pub family: String,
     pub weight: Option<u16>,
-    pub style: Option<String>,
+    pub style: SugarloafFontStyle,
 }
 
 impl Default for SugarloafFont {
@@ -14,7 +20,7 @@ impl Default for SugarloafFont {
         Self {
             family: default_font_family(),
             weight: None,
-            style: None,
+            style: SugarloafFontStyle::Normal,
         }
     }
 }
@@ -39,8 +45,8 @@ fn default_font_family() -> String {
 pub fn default_font_regular() -> SugarloafFont {
     SugarloafFont {
         family: default_font_family(),
-        weight: Some(400),
-        style: Some(String::from("normal")),
+        weight: Some(300),
+        style: SugarloafFontStyle::Normal,
     }
 }
 
@@ -48,15 +54,15 @@ pub fn default_font_bold() -> SugarloafFont {
     SugarloafFont {
         family: default_font_family(),
         weight: Some(800),
-        style: Some(String::from("normal")),
+        style: SugarloafFontStyle::Normal,
     }
 }
 
 pub fn default_font_italic() -> SugarloafFont {
     SugarloafFont {
         family: default_font_family(),
-        weight: Some(400),
-        style: Some(String::from("italic")),
+        weight: Some(300),
+        style: SugarloafFontStyle::Italic,
     }
 }
 
@@ -64,7 +70,7 @@ pub fn default_font_bold_italic() -> SugarloafFont {
     SugarloafFont {
         family: default_font_family(),
         weight: Some(800),
-        style: Some(String::from("italic")),
+        style: SugarloafFontStyle::Italic,
     }
 }
 
